@@ -2,7 +2,7 @@ package App::Daemon;
 use strict;
 use warnings;
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 use Getopt::Std;
 use Pod::Usage;
@@ -233,9 +233,11 @@ sub detach {
         # process group leader of a new process group.
     POSIX::setsid();
  
-    INFO "Process ID is $$";
-    pid_file_write($$);
-    INFO "Written to $pidfile";
+    if( defined $pidfile ) {
+        INFO "Process ID is $$";
+        pid_file_write($$);
+        INFO "Written to $pidfile";
+    }
 
     if($as_user) {
         user_switch();
@@ -416,7 +418,8 @@ sub pid_file_read {
     my $pid = <FILE>;
     chomp $pid if defined $pid;
     close FILE;
-    return $pid;
+    $pid =~ /^(\d+)$/; # Untaint
+    return $1;
 }
 
 ###########################################
@@ -700,6 +703,10 @@ After this, options C<-k>, C<-P>, and C<-U> will have disappeared from
 
 =head2 Gotchas
 
+=over 4
+
+=item Log File Permissions
+
 If the process is started as root but later drops permissions to a
 non-priviledged user for security purposes, it's important that 
 logfiles are created with correct permissions.
@@ -723,6 +730,29 @@ doesn't exist yet, and changes its owner to 'nobody'. Later, when the
 process assumes the identity of the user 'nobody', it will continue
 to write to the logfile without permission problems.
 
+=item Log4perl Categories
+
+Note that App::Daemon is logging messages in Log4perl's App::Daemon 
+namespace. So, if you're running your own Log4perl configuration and
+define a root logger like
+
+    log4perl.logger=DEBUG, appendername
+
+then App::Daemon's messages will bubble up to it and be visible in
+the output. If you don't want that, either use
+
+    log4perl.logger.My.App=DEBUG, appendername
+
+to explicitly enable verbose logging in your application namespace
+(and not in App::Daemon's) or tone down App::Daemon's verbosity via
+
+    log4perl.logger.App.Daemon=ERROR
+
+explicitly. If you want more details on basic Log4perl features,
+check out the L<Log::Log4perl> manual page.
+
+=back
+
 =head2 Detach only
 
 If you want to create a daemon without the fancy command line parsing
